from django.conf import settings
from django.db import models

from django_user_agents.utils import get_user_agent


def get_ip_address_from_request(request):
    meta_fields = ("HTTP_X_FORWARDED_FOR", "HTTP_X_REAL_IP", "REMOTE_ADDR")

    for field in meta_fields:
        value = request.META.get(field, "")
        if value:
            return value

    return ""


class BrowserLogDataManager(models.Manager):
    def log_user_agent(self, request):
        ip = get_ip_address_from_request(request)
        user = request.user if request.user.is_authenticated else None
        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)

        ua_string = request.META.get("HTTP_USER_AGENT", "")
        ua = get_user_agent(request)

        bld, _ = self.update_or_create(
            ip_address=ip,
            session_key=session_key,
            defaults=dict(
                ua_string=ua_string,
                browser_family=ua.browser.family,
                browser_version=ua.browser.version_string,
                os_family=ua.os.family,
                os_version=ua.os.version_string,
                device_family=ua.device.family,
                is_mobile=ua.is_mobile,
                is_tablet=ua.is_tablet,
                is_touch_capable=ua.is_touch_capable,
                is_pc=ua.is_pc,
                is_bot=ua.is_bot,
            ),
        )

        if user and not bld.user:
            bld.user = user
            bld.save()

        return bld


class BrowserLogData(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.CASCADE
    )
    session_key = models.CharField(max_length=40)
    ip_address = models.GenericIPAddressField()
    ua_string = models.TextField(blank=True, null=True)
    browser_family = models.CharField(max_length=50, blank=True, null=True)
    browser_version = models.CharField(max_length=20, blank=True, null=True)
    os_family = models.CharField(max_length=50, blank=True, null=True)
    os_version = models.CharField(max_length=20, blank=True, null=True)
    device_family = models.CharField(max_length=50, blank=True, null=True)
    is_mobile = models.BooleanField(default=False)
    is_tablet = models.BooleanField(default=False)
    is_touch_capable = models.BooleanField(default=False)
    is_pc = models.BooleanField(default=False)
    is_bot = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)

    objects = BrowserLogDataManager()

    def __str__(self):
        return "%s %s %s %s %s" % (
            self.ip_address,
            self.browser_family,
            self.browser_version,
            self.os_family,
            self.os_version,
        )
